"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = __importDefault(require("events"));
var RedisType;
(function (RedisType) {
    RedisType[RedisType["Integers"] = 0] = "Integers";
    RedisType[RedisType["Errors"] = 1] = "Errors";
    RedisType[RedisType["Strings"] = 2] = "Strings";
    RedisType[RedisType["BulkSrings"] = 3] = "BulkSrings";
    RedisType[RedisType["Array"] = 4] = "Array";
})(RedisType = exports.RedisType || (exports.RedisType = {}));
var RedisResponse = /** @class */ (function () {
    function RedisResponse(options) {
        this.type = options.type;
        this.intMsg = options.intMsg;
        this.errMsg = options.errMsg;
        this.strMsg = options.strMsg;
        this.bulkMsg = options.bulkMsg;
        this.arrMsg = options.arrMsg;
    }
    RedisResponse.prototype.toValue = function () {
        switch (this.type) {
            case RedisType.Errors: {
                return this.errMsg;
            }
            case RedisType.Array: {
                return this.arrMsg;
            }
            case RedisType.Integers: {
                return this.intMsg;
            }
            case RedisType.Strings: {
                return this.strMsg;
            }
            case RedisType.BulkSrings: {
                return this.bulkMsg;
            }
        }
    };
    return RedisResponse;
}());
exports.RedisResponse = RedisResponse;
var RedisCommand = /** @class */ (function (_super) {
    __extends(RedisCommand, _super);
    function RedisCommand(name) {
        var params = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            params[_i - 1] = arguments[_i];
        }
        var _this = _super.call(this) || this;
        _this.resolved = false;
        _this.name = name;
        _this.params = params;
        return _this;
    }
    RedisCommand.prototype.asPromise = function () {
        var _this = this;
        return new Promise(function (resolve) {
            if (_this.resolved) {
                resolve(_this.res);
                return;
            }
            else {
                _this.once("RedisResOK", function () {
                    _this.resolved = true;
                    resolve(_this.res);
                });
            }
        });
    };
    RedisCommand.prototype.setIntMsg = function (integer) {
        this.res = new RedisResponse({ type: RedisType.Integers, intMsg: integer });
        this.emit("RedisResOK");
    };
    RedisCommand.prototype.setErrMsg = function (errMsg) {
        this.res = new RedisResponse({ type: RedisType.Errors, errMsg: errMsg });
        this.emit("RedisResOK");
    };
    RedisCommand.prototype.setStrMsg = function (strMsg) {
        this.res = new RedisResponse({ type: RedisType.Strings, strMsg: strMsg });
        this.emit("RedisResOK");
    };
    RedisCommand.prototype.setBulkMsg = function (bulkMsg) {
        this.res = new RedisResponse({ type: RedisType.BulkSrings, bulkMsg: bulkMsg });
        this.emit("RedisResOK");
    };
    RedisCommand.prototype.setArr = function (arr) {
        this.res = new RedisResponse({ type: RedisType.Array, arrMsg: arr });
        this.emit("RedisResOK");
    };
    RedisCommand.prototype.toString = function () {
        var len = function (i) { return ("" + i).length; };
        var bytes = "*" + (this.params.length + 1) + "\r\n$" + len(this.name) + "\r\n" + this.name + "\r\n";
        this.params.forEach(function (e) {
            bytes += "$" + len(e) + "\r\n" + e + "\r\n";
        });
        return bytes;
    };
    return RedisCommand;
}(events_1.default));
function createCommand(name) {
    var params = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        params[_i - 1] = arguments[_i];
    }
    return new (RedisCommand.bind.apply(RedisCommand, [void 0, name].concat(params)))();
}
exports.createCommand = createCommand;
exports.default = RedisCommand;
