"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var RedisParser = /** @class */ (function () {
    function RedisParser(options) {
        this.options = options;
    }
    RedisParser.prototype.parse = function (data) {
        var lines = data.split("\r\n");
        while (lines.length) {
            var line = lines[0];
            var char = line[0];
            switch (char) {
                case ":": {
                    var interger = this.parseIntegers(line);
                    lines.shift();
                    this.options.onParseIntegers && this.options.onParseIntegers(interger);
                    break;
                }
                case "+": {
                    var data_1 = this.parseStrings(line);
                    lines.shift();
                    this.options.onParseStrings && this.options.onParseStrings(data_1);
                    break;
                }
                case "-": {
                    var message = this.parseErrors(line);
                    lines.shift();
                    this.options.onParseErrors && this.options.onParseErrors(message);
                    break;
                }
                case "$": {
                    var bulkString = this.parseBulkString(lines);
                    this.options.onParseBulkSrings && this.options.onParseBulkSrings(bulkString);
                    break;
                }
                case "*": {
                    var dataArr = this.parseArray(lines);
                    this.options.onParseArray && this.options.onParseArray(dataArr);
                    break;
                }
                default: {
                    lines.shift();
                }
            }
        }
    };
    RedisParser.prototype.parseArray = function (lines) {
        var numberData = lines.shift();
        var number = +String.prototype.slice.call(numberData, 1);
        var dataArr = [];
        while (number > 0) {
            var line = lines[0];
            var char = line[0];
            switch (char) {
                case ":": {
                    dataArr.push(this.parseIntegers(line));
                    lines.shift();
                    break;
                }
                case "+": {
                    dataArr.push(this.parseStrings(line));
                    lines.shift();
                    break;
                }
                case "$": {
                    dataArr.push(this.parseBulkString(lines));
                    break;
                }
            }
            number--;
        }
        return dataArr;
    };
    RedisParser.prototype.parseBulkString = function (lines) {
        var numberData = lines.shift();
        var number = String.prototype.slice.call(numberData, 1);
        if (+number >= 0) {
            var bulkString = lines.shift();
            return bulkString;
        }
        else {
            return ""; // should return null or undefine instead of empty string
            /**
             * This is called a Null Bulk String. The client library API should
             * not return an empty string, but a nil object, when the server replies
             *  with a Null Bulk String. For example a Ruby library should return 'nil'
             * while a C library should return NULL
             * (or set a special flag in the reply object), and so forth
             * */
        }
    };
    RedisParser.prototype.parseIntegers = function (line) {
        return +String.prototype.slice.call(line, 1);
    };
    RedisParser.prototype.parseErrors = function (line) {
        return String.prototype.slice.call(line, 5);
    };
    RedisParser.prototype.parseStrings = function (line) {
        return String.prototype.slice.call(line, 1);
    };
    return RedisParser;
}());
exports.default = RedisParser;
